/***************************************************************************
*   Copyright (C) 2003 by Hideki Ikemoto                                  *
*   ikemo@users.sourceforge.jp                                            *
*   linux 󥫥פ餰ο                                        *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/

#include "kitathreadview.h"

#include <kpopupmenu.h>
#include <krun.h>
#include <kurl.h>
#include <khtmlview.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kaction.h>
#include <kmessagebox.h>
#include <kdebug.h>
#include <kapplication.h>
#include <ktextbrowser.h>
#include <kcombobox.h>

#include <dom/html_inline.h>
#include <dom/html_base.h>
#include <dom/html_list.h>

#include <qclipboard.h>
#include <qlayout.h>
#include <qtoolbutton.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qprogressdialog.h>
#include <qcursor.h>
#include <qtextbrowser.h>
#include <qmessagebox.h>
#include <qdatetime.h>
#include <qeucjpcodec.h>
#include <qsimplerichtext.h>
#include <qpopupmenu.h>

#include <kxmlguifactory.h>
#include <kaction.h>
#include <kstdaccel.h>
#include <kstdaction.h>

#include "kitahtmlpart.h"
#include "kitawritedialog.h"
#include "kita2ch.h"
#include "kitanavi.h"

#include "libkita/kitaconfig.h"
#include "libkita/kita-utf8.h"
#include "libkita/threadinfo.h"
#include "libkita/qcp932codec.h"
#include "libkita/favoritethreads.h"
#include "libkita/favoriteboards.h"
#include "libkita/datmanager.h"
#include "libkita/parsemisc.h"
#include "libkita/board.h"
#include "libkita/signalcollection.h"
#include "libkita/kita_misc.h"

#define MAX_LABEL_LENGTH 60

static const char* cookie_title = "񤭹߳ǧ";
static const char* cookie_message =
    "Ƴǧ\n"
    "Ƥ줿Ƥϥԡ¸ѡž礬ޤ\n"
    "Ƥ˴ؤȯǤƼԤ˵ޤ\n"
    "\n"
    "Ǥ餦Ȥƽ񤭹ߤޤ\n";

KitaThreadView::KitaThreadView( QWidget* parent, const char* name )
        : KitaThreadViewBase( parent, name )
	  , KXMLGUIClient()
        , m_popup( 0 )
{
    m_thread = new Kita::NullThread();
    m_threadPart = new KitaHTMLPart( threadFrame );
    m_domtree = new KitaDomTree( m_threadPart );
    QHBoxLayout* aLayout = new QHBoxLayout( threadFrame );
    aLayout->addWidget( m_threadPart->view() );

    {
        subjectLabel = new Kita::KitaSubjectLabel( Subjectframe );
        QHBoxLayout* LBox = new QHBoxLayout( Subjectframe );
        LBox->addWidget( subjectLabel );
        connect( subjectLabel, SIGNAL( urlClick( const QString& ) ),
                 SLOT( subjectlinkClicked( const QString& ) ) );
        connect( subjectLabel, SIGNAL( urlMClick( const QString& ) ),
                 SLOT( subjectlinkMClicked( const QString& ) ) );
    }

    {
        SearchButton->setPixmap( SmallIcon( "find" ) );
        HighLightButton->setPixmap( SmallIcon( "idea" ) );
        ReloadButton->setPixmap( SmallIcon( "reload" ) );
        GobackAnchorButton->setPixmap( SmallIcon( "2leftarrow" ) );
        BookmarkButton->setPixmap( SmallIcon( "bookmark_add" ) );
        writeButton->setPixmap( SmallIcon( "pencil" ) );
    }

    setAcceptDrops( true ); // DND Drop eneble: 2nd stage. - enable on "KitaThreadView" widget and disable on the others(child widgets of "KitaThreadView").
    threadFrame->setAcceptDrops( false ); // don't treat Drop event on child.
    m_threadPart->view() ->setAcceptDrops( false ); // don't treat Drop event on child.

    m_threadPart->enableMetaRefresh( false ); //disable <meta refresh="...">


    connect( writeButton, SIGNAL( clicked() ),
             SLOT( slotWriteButtonClicked() ) );
    connect( m_threadPart, SIGNAL( nodeActivated( const DOM::Node& ) ),
             SLOT( slotDOMNodeActivated( const DOM::Node& ) ) );
    connect( m_threadPart, SIGNAL( onURL( const QString& ) ),
             SLOT( slotOnURL( const QString& ) ) );
    connect( m_threadPart, SIGNAL( setLocationBarURL( const QString& ) ),
             SIGNAL( setLocationBarURL( const QString& ) ) );
    connect( BookmarkButton, SIGNAL( toggled( bool ) ),
             SLOT( slotBookmarkButtonClicked( bool ) ) );
    connect( SearchButton, SIGNAL( clicked() ),
             SLOT( slotSearchButton() ) );
    connect( SearchCombo, SIGNAL( activated( int ) ),
             SLOT( slotSearchButton() ) );
    connect( GobackAnchorButton, SIGNAL( clicked() ),
             m_threadPart, SLOT( gobackAnchor() ) );
    connect( ReloadButton, SIGNAL( clicked() ),
             SLOT( slotReloadButton() ) );
    connect( gotoCombo, SIGNAL( activated( int ) ),
             SLOT( slotComboActivated( int ) ) );

    KParts::BrowserExtension* ext = m_threadPart->browserExtension();
    Kita::SignalCollection* signalCollection = Kita::SignalCollection::getInstance();
    connect( ext, SIGNAL( openURLRequest( const KURL&, const KParts::URLArgs& ) ),
             SLOT( slotOpenURLRequest( const KURL&, const KParts::URLArgs& ) ) );
    connect( this, SIGNAL( openURLRequest( const KURL&, const KParts::URLArgs& ) ),
             signalCollection, SIGNAL( openURLRequest( const KURL&, const KParts::URLArgs& ) ) );
    connect( this, SIGNAL( writeSucceeded() ),
             signalCollection, SIGNAL( writeSucceeded() ) );
    connect( ext, SIGNAL( createNewWindow ( const KURL&, const KParts::URLArgs& ) ),
             SIGNAL( createNewWindow ( const KURL&, const KParts::URLArgs& ) ) );
    connect( ext, SIGNAL( setLocationBarURL( const QString& ) ),
             SIGNAL( setLocationBarURL( const QString& ) ) );
    connect( ext, SIGNAL( enableAction( const char*, bool ) ),
             SIGNAL( enableAction( const char*, bool ) ) );
    connect( this, SIGNAL( signalChangeStatusbar( const QString& ) ),
             signalCollection, SIGNAL( signalChangeStatusbar( const QString& ) ) );
    connect( ext, SIGNAL( popupMenu( KXMLGUIClient*, const QPoint&, const KURL&, const QString&, mode_t ) ),
             this, SLOT( slotPopupMenu( KXMLGUIClient*, const QPoint&, const KURL&, const QString&, mode_t ) ) );
    connect( this, SIGNAL( bookmarked( const QString&, bool ) ),
             signalCollection, SIGNAL( bookmarked( const QString&, bool ) ) );
    connect( this, SIGNAL( showThreadCompleted( const KURL& ) ),
             signalCollection, SIGNAL( showThreadCompleted( const KURL& ) ) );
    connect( this, SIGNAL( signalThread( const Kita::Thread* ) ),
             signalCollection, SIGNAL( signalThread( const Kita::Thread* ) ) );
    connect( this, SIGNAL( openBoardRequested( const QString&, bool ) ),
             signalCollection, SIGNAL( openBoardRequested( const QString&, bool ) ) );
    
    setupActions();

    m_viewmode = VIEWMODE_PARENT;
    m_rescode = 200;
    m_serverTime = 0;
    m_datURL = QString::null;
}

KitaThreadView::~KitaThreadView()
{

    if ( m_popup )
    {
        delete m_popup;
        m_popup = NULL;
    }

    /* unlock URL */
    Kita::DatManager::unlock(m_datURL);

    /* don't delete m_threadPart before deleting m_domtree */
    if ( m_domtree )
    {
        delete m_domtree;
        m_domtree = NULL;
    }
    if ( m_threadPart )
    {
        delete m_threadPart;
        m_threadPart = NULL;
    }
}


/*---------------*/
/* setup actions */  /* private */
void KitaThreadView::setupActions(){
    
    setXMLFile("kitathreadviewui.rc");

    new KAction( i18n( "Find" ),
		 "find",
                 KStdAccel::find(),
                 this,
                 SLOT( focusSearchCombo() ),
                 actionCollection(),
                 "threadview_find" );
    
    new KAction( i18n( "Reload" ),
		 "reload",
                 KStdAccel::reload(),
                 this,
                 SLOT( slotReloadButton() ),
                 actionCollection(),
                 "threadview_reload" );
    
    new KAction(i18n("Stop"),
		"stop",
		Key_Escape,
		this,
		SLOT(slotStopLoading()),
	       actionCollection(),
		"threadview_stop");
}


const QString KitaThreadView::threadName() const
{
    return m_thread->name();
}

const KURL KitaThreadView::threadURL() const
{
    return m_thread->url();
}

void KitaThreadView::slotDOMNodeActivated( const DOM::Node& node )
{
    { //process Anchor tags. Anchor tags not proccessed here cause 'emit KParts::BrowserExtention::openURLRequest()'
        DOM::HTMLAnchorElement anchor = node;

        if ( ! anchor.href().isEmpty() )
        {
            kdDebug() << "AnchorNodeActivated::" << endl;
        } // end: anchor.href().isEmpty()
    } // end of Anchor tags.
}

void KitaThreadView::setSubjectLabel( const QString& boardName, const QString& threadName, const QString boardURL )
{
    QString disp;
    if ( boardName.isEmpty() )
    {
        disp = threadName;
    }
    else
    {
        disp = QString( "<a href=\"%1\">[%2]</a> %3" ).arg( boardURL ).arg( boardName ).arg( threadName );
    }

    //disp.truncate( MAX_LABEL_LENGTH );
    subjectLabel->setText( disp );
}

void KitaThreadView::subjectlinkClicked(const QString& URL )
{
    kdDebug() << QString("Jump board: \"%1\"").arg(URL) << endl;
    emit openBoardRequested( URL, KitaConfig::alwaysUseTab() );
}

void KitaThreadView::subjectlinkMClicked( const QString &URL )
{
    kdDebug() << QString("Jump board as new tab: \"%1\"").arg(URL) << endl;
    emit openBoardRequested( URL, TRUE );
}

void KitaThreadView::updateButton()
{
    writeButton->setEnabled( true );
    BookmarkButton->setEnabled( true );
    ReloadButton->setEnabled( true );

    if ( HighLightButton->isOn() )
    {
        HighLightButton->toggle();
    }
    if ( FavoriteThreads::getInstance() ->contains( m_thread->datURL() ) )
    {
        BookmarkButton->setOn( true );
    }
    else
    {
        BookmarkButton->setOn( false );
    }
}


void KitaThreadView::slotShowErrorDialog( const QString& input, const KURL& )
{
    QTextCodec * eucjpCodec = QTextCodec::codecForName( "euc-jp" );

    kdDebug() << "'" << input << "'" << endl;
    Kita::WriteResult writeResult( input );

    kdDebug() << "code = " << writeResult.code() << endl;
    switch ( writeResult.code() )
    {
    case Kita::K2ch_Unknown:
        // probably OK.
        emit writeSucceeded();
        slotReloadButton();
        break;
    case Kita::K2ch_True:
    case Kita::K2ch_False:
    case Kita::K2ch_Check:
        break;
    case Kita::K2ch_Error:
        {
            KMessageBox::error( 0, writeResult.message(), writeResult.title() );
            KitaWriteDialog* dialog = openDialog( m_postInfo );
            dialog->show();
        }
        break;
    case Kita::K2ch_Cookie:
        if ( KMessageBox::questionYesNo( 0, eucjpCodec->toUnicode( cookie_message ),
                                         eucjpCodec->toUnicode( cookie_title ) )
                == KMessageBox::Yes )
        {
            KitaWriteDialog * dialog = openDialog( m_postInfo );
            dialog->postMessage();
        }
        else
        {
            KitaWriteDialog* dialog = openDialog( m_postInfo );
            dialog->show();
        }
        break;
    default:
        break;
    }
}

KitaWriteDialog* KitaThreadView::openDialog( const Kita::PostInfo& info )
{
    KitaWriteDialog * new_dialog = KitaWriteDialog::open( info, m_thread );
    connect( new_dialog, SIGNAL( postStarted( KIO::Job*, const Kita::PostInfo& ) ),
             this, SLOT( slotPostStarted( KIO::Job*, const Kita::PostInfo& ) ) );
    connect( new_dialog, SIGNAL( postResponse( const QString&, const KURL& ) ),
             this, SLOT( slotShowErrorDialog( const QString&, const KURL& ) ) );
    connect( new_dialog, SIGNAL( postResponse( const QString&, const KURL& ) ),
             this, SIGNAL( postResponse( const QString&, const KURL& ) ) );
    return new_dialog;
}

/*--------------------*/
/* write response     */
/*--------------------*/  /* private slots */
void KitaThreadView::slotWriteButtonClicked( QString resstr )
{
    if ( ! m_serverTime ) return ;
    
    // TODO: machiBBS kakiko support.
    if ( Kita::boardType( m_thread->datURL() ) == Kita::Board_MachiBBS ) {
        QMessageBox::warning( this, QString("<(_ _)>"),
        i18n("Can't write to machi BBS in this version." ) );
        return;
    }

    KURL bbscgiURL = KURL( m_thread->boardURL(), "../test/bbs.cgi" );
    bbscgiURL.setProtocol( "http" );

    Kita::PostInfo info;
    info.host = bbscgiURL.host();
    info.bbs = m_thread->boardID();
    info.key = m_thread->datID();
    info.time = QString( "%1" ).arg( m_serverTime );
    info.body = resstr;
    info.mail = m_mailaddr;

    KitaWriteDialog* dialog = openDialog( info );
    dialog->show(); // work asynchronus.
}


void KitaThreadView::insertSearchCombo()
{
    for ( int count = 0; count < SearchCombo->count(); ++count )
    {
        if ( SearchCombo->text( count ) == SearchCombo->currentText() )
        {
            // found
            return ;
        }
    }
    SearchCombo->insertItem( SearchCombo->currentText() );
}

QStringList KitaThreadView::parseSearchQuery( const QString& input ) const
{
    QStringList tmp = QStringList::split( ' ', input );
    QStringList ret_list;
    QRegExp truncSpace( "\\s*$" );
    QStringList::iterator it = tmp.begin();
    for ( ; it != tmp.end(); ++it )
        ret_list += ( *it ).replace( truncSpace, "" );
    return ret_list;
}


void KitaThreadView::hidePopup()
{
    if ( m_popup )
    {
        m_popup->hide();
    }
}

void KitaThreadView::slotPopupMenu( KXMLGUIClient* client, const QPoint& global, const KURL& url, const QString& mimeType, mode_t mode )
{
    KActionCollection * collection = client->actionCollection();
    KAction* action;
    action = new KAction( i18n( "goback anchor" ), SmallIcon( "idea" ), KShortcut(), m_threadPart, SLOT( gobackAnchor() ), collection, "goback_anchor" );
    emit popupMenu( client, global, url, mimeType, mode );
}

void KitaThreadView::setFont( const QFont& font )
{
    //  m_threadPart->setStandardFont(font.family());
    subjectLabel->setFont( font );

    DOM::CSSStyleDeclaration style = m_threadPart->htmlDocument().body().style();
    style.setProperty( "font-family", font.family(), "" );
    style.setProperty( "font-size", QString( "%1pt" ).arg( font.pointSize() ), "" );
}

void KitaThreadView::slotPostStarted( KIO::Job*, const Kita::PostInfo& info )
{
    m_postInfo = info;
}

void KitaThreadView::slotBookmarkButtonClicked( bool on )
{
    emit bookmarked( m_thread->datURL(), on );
}


void KitaThreadView::focusSearchCombo()
{
    SearchCombo->setFocus();
}

namespace Kita
{

    KitaSubjectLabel::KitaSubjectLabel(QWidget* parent, const char * name )
            : KTextBrowser(parent, name)
    {
        setTextFormat(Qt::RichText);
        setVScrollBarMode(QScrollView::AlwaysOff);
        setHScrollBarMode(QScrollView::AlwaysOff);
        setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
        setNotifyClick( TRUE );
        setFrameStyle(QFrame::NoFrame);
        paletteChanged();
        if (kapp)
        {
            connect(kapp, SIGNAL(kdisplayPaletteChanged()),
                    this, SLOT(paletteChanged()));
        }
    }

    void KitaSubjectLabel::paletteChanged()
    {
        QPalette p = kapp ? kapp->palette() : palette();
        p.setBrush(QColorGroup::Base, p.brush(QPalette::Normal, QColorGroup::Background));
        p.setColor(QColorGroup::Text, p.color(QPalette::Normal, QColorGroup::Foreground));
        setPalette(p);
    }

    QSize KitaSubjectLabel::minimumSizeHint() const
    {
        QSize ms = minimumSize();
        if ((ms.width() > 0) && (ms.height() > 0))
            return ms;

        int w = 400;
        if (ms.width() > 0)
            w = ms.width();

        QString txt = text();
        QSimpleRichText rt(txt, font());
        rt.setWidth(w - 2*frameWidth() - 10);
        w = 10 + rt.widthUsed() + 2*frameWidth();
        if (w < ms.width())
            w = ms.width();
        int h = rt.height() + 2*frameWidth();
        if ( h < ms.height())
            h = ms.height();

        return QSize(w, h);
    }

    QSize KitaSubjectLabel::sizeHint() const
    {
        return minimumSizeHint();
    }
    void KitaSubjectLabel::contentsMousePressEvent ( QMouseEvent * e )
    {
        KTextBrowser::contentsMousePressEvent( e );
        kdDebug() << "Press Button:" << QString::number( e->button() ) << endl;
        if ( e->button() == Qt::MidButton )
        {
            QString t = text();
            QSimpleRichText rt( t, font() );
            m_popupURL = rt.anchorAt( e->pos() );
        }
    }

    void KitaSubjectLabel::contentsMouseReleaseEvent ( QMouseEvent * e )
    {
        KTextBrowser::contentsMouseReleaseEvent( e );
        if ( !m_popupURL.isEmpty() && e->button() == Qt::MidButton )
            emit urlMClick( m_popupURL );
    }

    QPopupMenu * KitaSubjectLabel::createPopupMenu ( const QPoint & pos )
    {
        QString t = text();
        QSimpleRichText rt( t, font() );
        m_popupURL = rt.anchorAt( pos );
        kdDebug() << QString( "popupMenu for \"%1\"").arg( m_popupURL ) << endl
;
        if ( !m_popupURL.isEmpty() )
        {
            QPopupMenu *popup = new QPopupMenu;
            popup->insertItem( i18n( "Open with new tab" ), this, SLOT( popupOpenAsNewTab() ) );
            popup->insertItem( i18n( "Copy title and URL" ), this, SLOT( popupCopyToClipboard() ) );
            QString ft = Kita::FavoriteBoards::boards().contains( KURL( m_popupURL ) ) ?
              i18n( "Remove from Favorites" ) : i18n( "Add to Favorites") ;
                popup->insertItem( ft, this, SLOT( popupFavorites() ) );
            return popup;
        }
        else return NULL;
    }

    void KitaSubjectLabel::popupOpenAsNewTab()
    {
        if ( !m_popupURL.isEmpty() )
            emit urlMClick( m_popupURL );
    }

    void KitaSubjectLabel::popupCopyToClipboard()
    {
        QClipboard * clipboard = QApplication::clipboard();
        QString text = QString("%1\n%2").arg( Kita::Board::getName( m_popupURL ) ).arg( m_popupURL );
        kdDebug() << "popupCopytoClipboard :" << endl << text << endl;
        clipboard->setText( text , QClipboard::Clipboard );
        clipboard->setText( text , QClipboard::Selection );
    }

    void KitaSubjectLabel::popupFavorites()
    {
        if ( !m_popupURL.isEmpty() ) {
            KURL u = KURL( m_popupURL );
            if ( Kita::FavoriteBoards::boards().contains( u ) )
                Kita::FavoriteBoards::remove( u );
            else
                Kita::FavoriteBoards::append( u );
        }
    }

    PopupTextBrowser::PopupTextBrowser( QFrame* frame )
            : QTextBrowser( frame )
    {
        zoomOut( static_cast<int>( pointSize() * 0.3 ) );
        setWordWrap( QTextEdit::NoWrap );
        setResizePolicy( QScrollView::AutoOne );
        setFont( KitaConfig::threadFont() );
        setPaletteForegroundColor( KitaConfig::popupColor() );
        setPaletteBackgroundColor( KitaConfig::popupBackgroundColor() );
    }

    PopupTextBrowser::~ PopupTextBrowser()
    {}

    ResPopup::ResPopup( KHTMLView* view )
            : QFrame( view, "res_popup", WStyle_Customize | WStyle_NoBorder | WStyle_Tool | WType_TopLevel | WX11BypassWM )
    {
        m_browser = new Kita::PopupTextBrowser( this );
    }

    ResPopup::~ ResPopup()
    {}

    void ResPopup::setText( QString& str )
    {
        m_browser->setText( str );
    }

    void ResPopup::adjustSize()
    {
        m_browser->resize( m_browser->contentsWidth() + 10, m_browser->contentsHeight() );
        QFrame::adjustSize();
    }

}


/*-------------------------------------------------*/
/* shobon extension                                */
/*-------------------------------------------------*/


/*-------*/
/* setup */
/*-------*/
void KitaThreadView::setupEx( const Kita::Thread* thread, int serverTime, int mode )
{
    /* config. */

    /* show m_preShowNum responses before 'kokomade yonda' */
    m_preShowNum = 50;

    /* max size of responses in the popup window */
    m_maxpopup = 10;

    /* default name */
    m_mailaddr = "sage";

    /* online mode */
    m_online = TRUE;


    /*---------------------------------------*/
    /* setup                                 */

    /* unlock previous URL */
    Kita::DatManager::unlock(m_datURL);

    m_thread = const_cast<Kita::Thread *>( thread );
    QString refstr;
    m_datURL = Kita::ParseMisc::parseURL(m_thread->datURL(),refstr);

    /* lock new URL */
    Kita::DatManager::lock(m_datURL);

    /* setup HTMLPart */
    m_threadPart->setupEx( this, m_domtree, m_thread );

    /* setup DOM */
    m_domtree->parseResInit( m_thread );

    /* mode. Mode is defined in kitathreadview.h */
    m_viewmode = mode;

    /* If curreent mode is VIEWMODE_PREVIEW , hide buttons */
    if ( m_viewmode == VIEWMODE_PREVIEW )
    {
        QLayout* Base = layout();
        Base->remove( Subjectframe );
        Base->removeItem( layout2 );
    }

    /* m_serverTime is used for writing */
    m_serverTime = serverTime;

    /* reset search direction */
    m_revsearch = FALSE;
}


/*--------------------------------------------------------*/
/* Show thread                                            */
/* This function is called from KitaThreadTabWidget class */
/*--------------------------------------------------------*/
void KitaThreadView::showThread( const Kita::Thread* thread2h )
{

    /* If this widget is not parent, then do nothing. */
    if ( m_viewmode != VIEWMODE_PARENT ) return ;

    if ( topLevelWidget() ->isMinimized() ) topLevelWidget() ->showNormal();
    topLevelWidget() ->raise();
    setActiveWindow();

    /* setup */
    setupEx( thread2h , 0, VIEWMODE_PARENT );

    /* get log from cahce */
    int maxnum = Kita::DatManager::getMaxResNumber(m_datURL);
    m_rescode = 200;
    if ( maxnum )
    {
        /* show templates (No.1->No.KitaDomTree::m_templateNum) */
        m_domtree->appendTemplate();

        /* rendering */
        m_kokoyonNum = Kita::DatManager::getKokoyonNum(m_datURL);
        for ( int i =  m_kokoyonNum - m_preShowNum ; i <= m_kokoyonNum ; i++ )
            m_domtree->appendRes( i, FALSE );
        updateScreen();

        m_threadPart->gotoKokoyon();
    }
    else
    {
        m_kokoyonNum = 0;
        showStatusBar( "" );
    }

    /* update data */
    updateInfo();
    m_firstReceive = TRUE;
    if( m_online ) slotReloadButton();

    topLevelWidget() ->raise();
    setActiveWindow();
}



/*---------*/
/* reload  */
/*---------*/ /* public slot */
void KitaThreadView::slotReloadButton()
{
    switch ( m_viewmode )
    {

    case VIEWMODE_PREVIEW: break;

    case VIEWMODE_PARENT:

        /* update cache */
        /* After updating cache, DatManager will call slotFinishLoad.*/
        if( m_online )
        {
            m_domtree->StopParseThread();
            if ( KitaConfig::afterShowNum() != 0 ) {
                m_showNum = m_domtree->getBottomResNumber() + KitaConfig::afterShowNum();
            } else {
                // TODO: fix it.
                m_showNum = 5000;
            }
            Kita::DatManager::updateCache(m_datURL,this);
            m_threadPart->view()->setFocus();
            showStatusBar( Kita::ParseMisc::utf8ToUnicode( KITAUTF8_NOWRENEW ) );
        }
        break;

    default:

        /* When this is not parent, show the thread with new tab */
        KParts::URLArgs tmp;
        emit openURLRequest( m_thread->datURL() , tmp );
        break;

    }
}

/*-----------------------------------*/
/* slot called when Kita::DatManager
   received new data.                */ /* public slot */
void KitaThreadView::slotReceiveData()
{

    int bottomNum = m_domtree->getBottomResNumber();
    int afterNum = Kita::DatManager::getMaxResNumber(m_datURL);

    m_rescode = Kita::DatManager::getResponseCode(m_datURL);
    m_serverTime = Kita::DatManager::getServerTime(m_datURL);

    /* rendering */
    if( (m_firstReceive && bottomNum < afterNum )
	|| (bottomNum + 20 < afterNum && m_showNum +2 >= afterNum ) )
    {
        for ( int i =  bottomNum+1 ; i <= QMIN( m_showNum, afterNum ) ; i++ ) m_domtree->appendRes( i, FALSE );
        updateScreen();
        if(m_firstReceive  && m_kokoyonNum < afterNum )
        {
            m_threadPart->gotoKokoyon();
            m_firstReceive = FALSE;
        }
    }
}

/*-----------------------------------*/
/* slot called when Kita::DatManager
   finished updating new data.       */ /* public slot */
void KitaThreadView::slotFinishLoad()
{

    m_firstReceive = FALSE;
    int bottomNum = m_domtree->getBottomResNumber();
    for ( int i =  bottomNum+1 ; i <= m_showNum; i++ ) m_domtree->appendRes( i, FALSE );
    updateScreen();
    updateInfo();
}

/*-----------------------------------*/
/* stop loading the thread           */ /* public slot */
void KitaThreadView::slotStopLoading(){
    Kita::DatManager::stopLoading(m_datURL);
}


/*---------------------------------------------------*/
/* This function is called when user clicks the link */
/*---------------------------------------------------*/  /* public slot */
void KitaThreadView::slotOpenURLRequest( const KURL& urlin, const KParts::URLArgs& args )
{
    KURL url = urlin;

    /* convert to the correct URL */
    /* see also comments in KitaDomTree::createHTMLDocument for details */
    if ( urlin.protocol() == "file" )
    {

        if ( urlin.prettyURL().mid( 5, 14 ) == "/test/read.cgi" )
            url = KURL( m_thread->datURL(), ".." + urlin.prettyURL().mid( 5 ) );
        else
            url = KURL( m_thread->datURL(), urlin.prettyURL().mid( 15 ) );
    }

    KURL datURL = filterReadCGI( url , m_thread );

    /*  If this is not anchor, then emit openURLRequest and return */
    if ( datURL.host() != KURL( m_thread->datURL() ).host()
            || datURL.path() != KURL( m_thread->datURL() ).path() )
    {
        emit openURLRequest( datURL, args );
        return ;
    }

    if ( ! datURL.hasRef() ) return ;
    if ( m_viewmode == VIEWMODE_PREVIEW ) return ;

    /*---------------------------*/
    /* show popupmenu for #write */

    if ( datURL.ref().left( 5 ) == "write" )
    {

        QClipboard * clipboard = QApplication::clipboard();
        QString str, resstr;
        int resNum = datURL.ref().mid( 5 ).toInt();

        /* show res tree on the popup */
        if(m_threadPart->isRightClick())
        {
            int num;
            QString htmlstr
            = Kita::DatManager::getTreeByRes(m_thread->datURL(),resNum,num);
            if ( !num ) return ;
            QString tmpstr = QString("<DIV>No.%1 : [%2]<BR>").arg(resNum).arg(num);
            tmpstr += htmlstr;
            tmpstr += "<BR><BR></DIV>";
            showPopup( tmpstr, QString::null );
            return;
        }
        /* show tree on the kitanavi */
        else if(m_threadPart->isCtrlClick() || m_threadPart->isMidClick())
        {
            KitaNavi::showKitaNaviResTree(m_datURL,resNum);
            return;
        }

        KPopupMenu *popupMenu = new KPopupMenu( m_threadPart->view() );
        popupMenu->clear();
        popupMenu->insertItem( i18n( "write response" ), 0 );
        popupMenu->insertItem( i18n( "quote this" ), 1 );
        popupMenu->insertItem( i18n( "copy" ), 3 );
        if ( m_viewmode == VIEWMODE_PARENT )
        {
            popupMenu->insertSeparator();
            popupMenu->insertItem( i18n( "set Kokomade Yonda" ), 2 );
        }

        switch ( popupMenu->exec( QCursor::pos() ) )
        {
        case 0:
            resstr = ">>" + QString().setNum( resNum ) + "\n";
            break;

        case 1:
            resstr = ">>" + QString().setNum( resNum ) + "\n";
            resstr += m_domtree->getResStr( resNum, "> " );
            break;

        case 2:
            delete popupMenu;
            m_kokoyonNum = resNum;
            Kita::DatManager::setKokoyonNum(m_datURL,resNum);
            updateScreen();
            gotoAnchor( QString().setNum( resNum ) );
            return ;

        case 3:
            delete popupMenu;
            str = threadName()
                  + "\n" + threadURL().url() + QString().setNum( resNum )
                  + "\n\n" + m_domtree->getResStr( resNum, "" );
            clipboard->setText( str , QClipboard::Clipboard );
            clipboard->setText( str , QClipboard::Selection );
            return ;

        default:
            delete popupMenu;
            return ;
        }

        delete popupMenu;
        slotWriteButtonClicked( resstr );
        return ;
    }

    /*----------------------------*/
    /* extract responses by ID    */

    if(datURL.ref().left(5) == "idpop")
    {

        QString strid = datURL.ref().mid(5).replace("%2B","+").replace("%2F", "/"); /* decode %2B -> +, %2F -> / */

        /* show them on the popup */
        if(m_threadPart->isRightClick())
        {
            int num;
            QString htmlstr
            = Kita::DatManager::getHtmlByID(m_thread->datURL(),strid,num );
            if ( num <= 1 ) return ;
            QString tmpstr = QString("<DIV>ID:%1:[%2]<BR>").arg(strid).arg(num);
            tmpstr += htmlstr;
            tmpstr += "<BR><BR></DIV>";
            showPopup( tmpstr, QString::null );
        }

        /* show them on the kitanavi */
        else if(m_threadPart->isCtrlClick() || m_threadPart->isMidClick())
            KitaNavi::showKitaNaviByID(m_datURL,strid);
        else
        {
            KPopupMenu *popupMenu = new KPopupMenu( m_threadPart->view() );
            popupMenu->clear();
            popupMenu->insertItem( i18n( "add id to abone list" ), 0 );
            switch ( popupMenu->exec( QCursor::pos() ) )
            {
            case 0:
                KitaConfig::addAboneID( strid );
                delete popupMenu;
                break;
            default:
                delete popupMenu;
                break;
            }
        }

        return;
    }

    /*----------------------------*/
    /* next 100 ,before 100 ,etc. */

    int top = m_domtree->getTopResNumber();
    int bottom = m_domtree->getBottomResNumber();
    int totalNum = Kita::DatManager::getMaxResNumber(m_datURL);

    if ( datURL.ref().left( 7 ) == "tugi100" )
    {

        if ( totalNum != bottom )
        {
            appendRes( bottom + 1, bottom + 100 );
        }
        gotoAnchor( QString().setNum( bottom ) );
        return ;

    }
    else if ( datURL.ref().left( 6 ) == "nokori" )
    {

        if ( totalNum != bottom )
        {
            appendRes( bottom + 1, totalNum );
        }
        gotoAnchor( QString().setNum( bottom ) );
        return ;

    }
    else if ( datURL.ref().left( 7 ) == "tosaigo" )
    {

        if ( totalNum != bottom )
        {
            appendRes( bottom + 1, totalNum );
        }
        gotoAnchor( "footer" );
        return ;

    }
    else if ( datURL.ref().left( 6 ) == "mae100" )
    {

        if ( top != 1 )
        {
            m_domtree->appendTemplate();
            appendRes( top - 100, bottom );
            gotoAnchor( QString().setNum( top ) );
        }
        return ;

    }
    else if ( datURL.ref().left( 8 ) == "maezenbu" )
    {

        if ( top != 1 )
        {
            appendRes( 1, bottom );
            gotoAnchor( QString().setNum( top ) );
        }
        return ;

    }
    else if ( datURL.ref().left( 6 ) == "tmp100" )
    {

        int tmpnum = m_domtree->getTemplateNumber();

        if ( tmpnum < top )
        {
            m_domtree->setTemplateNumber( tmpnum + 100 );
            m_domtree->appendTemplate();
            appendRes( top, bottom );
            gotoAnchor( QString().setNum( tmpnum ) );
        }
        return ;

    }
    else if ( datURL.ref().left( 5 ) == "zenbu" )
    {

        if ( top != 1 || bottom != totalNum )
        {
            appendRes( 1, totalNum );
            gotoAnchor( "header" );
        }
        return ;

    }

    /*---------------------*/
    /* jump to anchor      */

    QString refstr = datURL.ref();
    int refNum,refNum2;

    int i = refstr.find( "-" );
    if ( i != -1 )
    {
        refNum = refstr.left( i ).toInt();
        refNum2 = QMIN(refstr.mid(i+1).toInt(),totalNum);
        if(refNum2 < refNum) refNum2 = refNum;
        refstr = datURL.ref().left( i );
    }
    else refNum = refNum2 = refstr.toInt();

    if ( !refNum ) return ;

    /* show Kita Navi or goto anchor */
    if(m_threadPart->isCtrlClick() || m_threadPart->isMidClick()
            || m_viewmode == VIEWMODE_KITANAVI)
    {
        KitaNavi::showKitaNavi(m_datURL,refNum,refNum2);
    }
    else gotoAnchor( refstr );
}



/*-----------------------------------------*/
/* show the information at the statusbar.  */
/*-----------------------------------------*/
void KitaThreadView::showStatusBar( QString info )
{
    int totalNum = 0, datSize = 0;;
    bool broken = FALSE;
    QString infostr = QString::null;
    QString errstr = QString::null;

    switch( m_viewmode )
    {

    case VIEWMODE_PARENT:

        totalNum = Kita::DatManager::getMaxResNumber(m_datURL);
        broken = Kita::DatManager::isBroken(m_datURL);
        datSize =  Kita::DatManager::getDatSize(m_datURL);

        errstr = QString::null;
        if ( m_rescode != 200 && m_rescode != 206 )
            errstr = QString( "Error %1" ).arg( m_rescode );
        if ( broken ) info += " This thread is broken.";

        /* show status bar */
        infostr = m_thread->name() +
                  QString( " [Total: %1 New: %2] %3 k" ).arg( totalNum ).arg( totalNum - m_kokoyonNum ).arg( datSize / 1024 )
                  + info + " " + errstr;

        emit signalChangeStatusbar( infostr );
        emit showThreadCompleted( m_thread->url() );
        topLevelWidget() ->setCaption( m_thread->name() );

        return;
        break;

    case VIEWMODE_KITANAVI:

        QString infostr = QString( "[%1] %2" ).arg(m_thread->boardName()).arg( m_thread->name() );
        emit signalChangeStatusbar( infostr );

        return;
        break;
    }
}



/* TODO: need to write test case. */  /* !!! public & static !!! */
KURL KitaThreadView::filterReadCGI( const KURL& url, const Kita::Thread* thread )
{
    KURL newURL = url;

    if ( url.path().contains( "/test/read.cgi" ) )
    {
        newURL.setProtocol( KURL( thread->datURL() ).protocol() );
        QString tmp = url.path().section( "/test/read.cgi", 1 );

        QString newPath = QString( "/%1/dat/%2.dat" )
                          .arg( tmp.section( '/', 1, 1 ) )
                          .arg( tmp.section( '/', 2, 2 ) );
        newURL.setPath( newPath );

        QString refBase = tmp.section( '/', 3 );
        if ( ! refBase.isEmpty() )
        {

            QString newRef;
            if ( refBase.at( 0 ) == '-' ) newRef = "1" + refBase;
            else newRef = refBase;

            newURL.setRef( newRef );

        }
    }
    kdDebug() << "newURL: " << newURL.url() << endl;
    return newURL;
}





/*----------------------*/
/* move to anchor       */
/*----------------------*/
void KitaThreadView::gotoAnchor( QString anc )
{

    int top = m_domtree->getTopResNumber();
    int bottom = m_domtree->getBottomResNumber();
    int res = anc.toInt();

    if ( res == 1 )
    {
        anc = "header";
    }
    else if ( res > 1 )
    {

        /* data is not set */
        if ( !m_domtree->isResDataSet( res ) ) return ;

        /* show res if it is not shown */
        if ( !m_domtree->isResShown( res ) )
        {

            if ( res > bottom ) appendRes( bottom + 1, res + 99 );
            else if ( res < top )
            {
                m_domtree->appendTemplate();
                appendRes( res, bottom );
            }
        }

        /* aboned */
        if ( !m_domtree->isResShown( res ) )
        {
            while ( res > 0 )
            {
                if ( m_domtree->isResShown( res ) ) break;
                res--;
            }
        }

        anc = QString().setNum( res );
    }

    m_threadPart->gotoAnchor( anc );
}


/*-------------------*/
/* update screen     */ /* public */
void KitaThreadView::updateScreen()
{
    int totalNum = Kita::DatManager::getMaxResNumber(m_datURL);

    /* set header, footer , etc. */
    m_domtree->appendMae100();
    m_domtree->appendTugi100();
    m_domtree->setKokoyonNum( m_kokoyonNum );
    m_domtree->appendKokoyon();
    m_domtree->appendFooter( totalNum );

    /* update display */
    m_threadPart->htmlDocument().applyChanges();
    m_threadPart->view() ->layout();
    m_threadPart->view() ->setVScrollBarMode( QScrollView::AlwaysOn );

}


/*--------------------*/
/* update information */ /* public */
void KitaThreadView::updateInfo()
{

    /* uptate informations */
    setSubjectLabel(Kita::DatManager::thread_boardName(m_datURL),
                    Kita::DatManager::thread_name(m_datURL)
                    + QString( " (%1)" )
                    .arg( Kita::DatManager::getMaxResNumber(m_datURL) ),
                    Kita::DatManager::thread_boardURL( m_datURL ) );
    emit showThreadCompleted( Kita::DatManager::thread_url(m_datURL) );
    m_domtree->findTextInit();
    updateButton();

    gotoCombo->clear();
    gotoCombo->insertItem( Kita::ParseMisc::utf8ToUnicode( KITAUTF8_GOTO ) );
    gotoCombo->insertItem( Kita::ParseMisc::utf8ToUnicode( KITAUTF8_KOKOYON ) );
    for( int i = 1; i < m_thread->resNum(); i += 100 )
    {
        gotoCombo->insertItem( QString().setNum( i ) + "-" );
    }
    gotoCombo->insertItem( Kita::ParseMisc::utf8ToUnicode( KITAUTF8_SAIGO ) );
    gotoCombo->adjustSize();

    emit signalThread( m_thread );
    showStatusBar( "" );
    m_threadPart->view() ->setFocus();
    m_domtree->parseAllRes();
}



/*-------------------*/
/* append  responses */
/*-------------------*/
void KitaThreadView::appendRes( int startnum, int endnum )
{

    showStatusBar( Kita::ParseMisc::utf8ToUnicode( KITAUTF8_NOWEXPAND ) );

    /* show clock cursor */
    QCursor qc; qc.setShape( Qt::WaitCursor );
    QApplication::setOverrideCursor( qc );

    for ( int i = startnum ; i <= endnum; i++ ) m_domtree->appendRes( i, FALSE );

    updateScreen();
    showStatusBar( "" );
    QApplication::restoreOverrideCursor();
}

/*---------------------*/
/* open popup window   */
/*---------------------*/
void KitaThreadView::showPopup( QString innerHTML, QString imgfile )
{

    if ( m_popup )
    {
        delete m_popup;
        m_popup = NULL;
    }

    /* text popup */
    if ( imgfile == NULL )
    {
        m_popup = new Kita::ResPopup( m_threadPart->view() );
        m_popup->setText( innerHTML );
        m_popup->adjustSize();
    }

    connect( m_threadPart->view(), SIGNAL( leave() ), SLOT( hidePopup() ) );

    QRect qr = QApplication::desktop() ->rect();
    int sw = qr.width(), sh = qr.height();
    int pw = m_popup->width(), ph = m_popup->height();
    const int mrg = 10;

    QPoint pos = QCursor::pos();
    int x = pos.x(), y = pos.y();

    if ( x + mrg + pw < sw )
    {
        x = x + mrg;
        y = QMAX( 0, y - ( ph + mrg ) );
    }
    else if ( y - ( mrg + ph ) >= 0 )
    {
        x = QMAX( 0, sw - pw );
        y = y - ( ph + mrg );
    }
    else if ( y + ( mrg + ph ) <= sh )
    {
        x = QMAX( 0, sw - pw );
        y = y + mrg;
    }
    else if ( x - ( mrg + pw ) >= 0 )
    {
        x = x - ( mrg + pw );
        y = QMAX( 0, y - ( ph + mrg ) );
    }
    else
    {
        x = x + mrg;
        y = QMAX( 0, y - ( ph + mrg ) );
    }

    pos.setX( x );
    pos.setY( y );

    m_popup->move( pos );
    m_popup->show();

}

/*------------------------------------------------------*/
/* This function is called when the mouse is on the URL */
/*------------------------------------------------------*/ /* private slots */
void KitaThreadView::slotOnURL( const QString& url )
{
    hidePopup();
    if ( !isActiveWindow() ) return ;
    if ( url.isEmpty() ) return ;

    if ( m_popup )
    {
        delete m_popup;
        m_popup = NULL;
    }

    /* id popup */
    if ( url.left( 6 ) == "#idpop" )
    {
        int num = Kita::DatManager::getNumByID( m_thread->datURL(), url.mid( 6 ) );
        if ( num <= 1 ) return ;
        QString tmpstr = QString( "<DIV>ID:%1:[%2]<BR><BR></DIV>" ).arg( url.mid( 6 ) ).arg( num );
        showPopup( tmpstr, QString::null );
        return ;
    }

    /*-------------------------*/
    /* popup for anchor        */

    QString innerHTML = QString::null;
    QString refstr;
    KURL datURL = m_thread->datURL();
    int refNum;
    int refNum2;

    if ( url.at( 0 ) == '#' ) refstr = url.mid( 1 );
    else if ( url.left( 7 ) == "mailto:" )
    {
        DOM::Node node = m_threadPart->nodeUnderMouse().firstChild();
        QString nodeValue = node.nodeValue().string();
        refstr = nodeValue;
    }
    else
    {
        datURL = filterReadCGI( KURL( m_thread->datURL(), url ) , m_thread );
        refstr = datURL.ref();
    }

    /* Is this url 2ch ? */
    /* copied from KitaThreadTabWidget::slotOpenURLRequest */
    QRegExp regexp( "http://([^/]*)/([^/]*)/dat/(.*)\\.dat" );
    if ( regexp.search( datURL.url() ) == -1 ) {
	if( Kita::boardType( datURL.url() ) != Kita::Board_MachiBBS )
	return ;
    }

    int i = refstr.find( "-" );
    if ( i != -1 )
    { /* >>refNum-refNum2 */

        refNum = refstr.left( i ).toInt();
        refNum2 = refstr.mid( i + 1 ).toInt();

        if ( refNum )
        {
            if ( refNum2 < refNum ) refNum2 = refNum;
            if ( refNum2 - refNum > m_maxpopup - 1 ) refNum2 = refNum + m_maxpopup - 1;
        }

    }
    else
    { /* >>refNum */
        refNum = refstr.toInt();
        refNum2 = refNum;
    }

    /* another thread ? */
    if ( datURL.host() != KURL( m_thread->datURL() ).host()
            || datURL.path() != KURL( m_thread->datURL() ).path() )
    {

        /* show subject */
        innerHTML = Kita::DatManager::getSubject( datURL );
        if ( innerHTML != QString::null ) innerHTML += "<br><br>";

        if ( !refNum ) refNum = refNum2 = 1;
    }

    /* get HTML and show it */
    if ( !refNum ) return ;
    innerHTML += Kita::DatManager::getHtml( datURL, refNum, refNum2 );

    if ( innerHTML != QString::null ) showPopup( innerHTML, QString::null );
}


/*------------------------*/
/* search function        */
/*------------------------*/  /* private slots */
void KitaThreadView::slotSearchButton()
{
    if ( m_viewmode == VIEWMODE_PREVIEW ) return ;

    if ( m_datURL.isEmpty() ) return ; /* Nothing is shown on the screen.*/

    /* jump */
    QString str = SearchCombo->currentText();
    if ( str.at( 0 ) == ':' )
    {
        QString anc = str.mid( 1 );
        gotoAnchor( anc );
        SearchCombo->setFocus();
        return ;
    }

    /* extract responses by keywords */
    else if(str.at(0) == '?')
    {
        QString query = str.mid(1);
        KitaNavi::showKitaNaviByWord(m_datURL,query);
        return;
    }

    /* if this is parent, then show all responses, and search */
    if ( m_viewmode == VIEWMODE_PARENT )
    {
        int top = m_domtree->getTopResNumber();
        int bottom = m_domtree->getBottomResNumber();
        int totalNum = m_domtree->getMaxResNumber();
        if ( top != 1 || bottom != totalNum )
        {
            appendRes( 1, totalNum );
        }
    }

    insertSearchCombo();
    QStringList list = parseSearchQuery( SearchCombo->currentText() );
    m_domtree->findText( SearchCombo->currentText(), m_revsearch );
    SearchCombo->setFocus();
}
// vim:sw=2:

void KitaThreadView::slotComboActivated( int index )
{
    if ( index == gotoCombo->count() - 1 )
    {
        // last
        gotoAnchor( "footer" );
    }
    else if ( index == 1 )
    {
        // kokomade yonda
        gotoAnchor( "kokomade_yonda" );
    }
    else if ( index != 0 )
    {
        QString numText = gotoCombo->text( index );
        numText.truncate( numText.length() - 1 );
        gotoAnchor( numText );
    }
}

/*--------------------------------------------------------*/
/* These public functions are used for KitaNavi & preview */
/*--------------------------------------------------------*/

/*---------------------*/
/* set data & parse it */
/*---------------------*/ /* public */
void KitaThreadView::domAppendData( QString& str, int bufnum , int num )
{
    m_domtree->appendRes( bufnum, FALSE );
}


/* public */
void KitaThreadView::domDeleteRes( int bufnum )
{
    m_domtree->removeRes( bufnum );
}


/* public */
void KitaThreadView::domAppendBelt( QString idstr )
{
    m_domtree->createBeltNode( idstr );
}

/* public */
void KitaThreadView::domSetInnerHTML(const QString& html )
{
    m_threadPart->htmlDocument().body().setInnerHTML(html);
    KitaThreadViewBaseLayout->remove( subjectLabel );
    KitaThreadViewBaseLayout->removeItem( layout2 );
    showStatusBar( "" );
}

/* public */
void KitaThreadView::domApplyChange( QString lbstr,  /* label */
                                     QString hstr,   /* header */
                                     QString fstr,   /* footer */
                                     QString anchor,  /* anchor to jump */
                                     QString cmbstr /* strings to set combo box */
                                   )
{

    /* label, header & footer */
    if ( hstr != NULL ) m_domtree->createCommentNode( hstr, "header", 0, 2, TRUE );
    if ( fstr != NULL ) m_domtree->createCommentNode( fstr, "footer", 0, 0, FALSE );
    if ( lbstr != NULL ) subjectLabel->setText( lbstr );

    /* update display */
    m_threadPart->htmlDocument().applyChanges();
    m_threadPart->view() ->layout();
    m_threadPart->view() ->setVScrollBarMode( QScrollView::AlwaysOn );
    m_threadPart->view() ->setFocus();

    GobackAnchorButton->setEnabled( FALSE );
    BookmarkButton->setEnabled( FALSE );
    switch( m_viewmode )
    {

    case VIEWMODE_PREVIEW:
        writeButton->setEnabled( FALSE );
        SearchCombo->setEnabled( FALSE );
        SearchButton->setEnabled( FALSE );
        HighLightButton->setEnabled( FALSE );
        ReloadButton->setEnabled( FALSE );
        break;

    case VIEWMODE_KITANAVI:
        writeButton->setEnabled( TRUE );
        ReloadButton->setEnabled( TRUE );
        break;
    }

    if ( cmbstr != NULL ) SearchCombo->insertItem( cmbstr );
    if ( anchor != NULL ) gotoAnchor( anchor );

    showStatusBar( "" );
}
