/***************************************************************************
*   Copyright (C) 2003 by Hideki Ikemoto                                  *
*   ikemo@users.sourceforge.jp                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/

#ifndef _KITATHREADVIEW_H_
#define _KITATHREADVIEW_H_

#include "kitathreadviewbase.h"

#include <kparts/browserextension.h>
#include <dom/html_base.h>
#include <kxmlguiclient.h>

#include <qtextbrowser.h>
#include <ktextbrowser.h>

#include "kita2ch.h"
#include "libkita/thread.h"
#include "kitadomtree.h"

class KURL;
class KitaHTMLPart;
class KitaWriteDialog;

#include <qvaluestack.h> // for gcc-2.9x
template <class T> class QValueStack;

namespace Kita
{
    class Access;
    class KitaSubjectLabel;
    class PopupTextBrowser;
    class ResPopup;
}

/* mode , m_viewmode uses them. */
enum{
    VIEWMODE_PARENT,
    VIEWMODE_KITANAVI,
    VIEWMODE_PREVIEW
};

namespace KIO
{
    class Job;
}

/**
 *
 * @author Hideki Ikemoto <ikemo@users.sourceforge.jp>
 **/

class KitaThreadView : public KitaThreadViewBase, public KXMLGUIClient
{
    Q_OBJECT

public:
    KitaThreadView( QWidget* parent, const char* name = 0 );
    ~KitaThreadView();
    const QString threadName() const;
    const KURL threadURL() const;

    void setupEx( const Kita::Thread* thread, int serverTime, int mode );
    void showStatusBar( QString info );
    void gotoAnchor( QString anc );
    Kita::Thread* get2chThread() { return m_thread; }
    int getViewMode() { return m_viewmode; }

    /* !! static !! */
    static KURL filterReadCGI( const KURL&, const Kita::Thread* thread );

    /* for preview and KitaNavi */
    void domAppendData( QString& str, int bufnum, int num );
    void domDeleteRes( int bufnum );
    void domAppendBelt( QString idstr );
    void domSetInnerHTML(const QString& html );
    void domApplyChange( QString lbstr, QString hstr, QString fstr, QString anchor, QString cmbstr );

public slots:
    void subjectlinkClicked(const QString& URL );
    void subjectlinkMClicked( const QString &URL );
    void showThread( const Kita::Thread* thread );
    void setFont( const QFont& font );
    void slotReloadButton();
    void slotOpenURLRequest( const KURL&, const KParts::URLArgs& );
    void slotReceiveData();
    void slotFinishLoad();
    void slotStopLoading();        
    void focusSearchCombo();
    
protected slots:
    void slotDOMNodeActivated( const DOM::Node& node );
    void slotPopupMenu( KXMLGUIClient*, const QPoint&, const KURL&, const QString&, mode_t );

private:        // Private attributes
    int m_serverTime;
    Kita::Thread* m_thread;
    KURL m_datURL;
    KitaHTMLPart* m_threadPart;
    Kita::PostInfo m_postInfo;
    Kita::ResPopup* m_popup;
    Kita::KitaSubjectLabel* subjectLabel;

    KitaDomTree* m_domtree;
    bool m_revsearch;
    int m_viewmode;
    int m_rescode;
    int m_kokoyonNum;
    int m_showNum;

    /* for config */
    int m_preShowNum;
    QString m_mailaddr;
    int m_maxpopup;
    bool m_online;
    bool m_firstReceive;

    void setupActions();
	
    void updateScreen();
    void updateInfo();

    void appendRes( int startnum, int endnum );
    void showPopup( QString innerHTML, QString imgfile );

    void insertSearchCombo();
    void setSubjectLabel( const QString& boardName, const QString& threadName, const QString boardURL );
    KitaWriteDialog* openDialog( const Kita::PostInfo& info );
    QStringList parseSearchQuery( const QString& input ) const;
    void updateButton();


private slots:
    void slotSearchButton();
    void slotOnURL( const QString& url );
    void slotPostStarted( KIO::Job*, const Kita::PostInfo& );
    void slotBookmarkButtonClicked( bool on );
    void slotWriteButtonClicked( QString resstr = QString::null );
    void slotShowErrorDialog( const QString& input, const KURL& urlOfInput = 0 );
    void slotComboActivated( int index );
    void hidePopup();

signals:
    void showThreadCompleted( const KURL& threadUrl );
    void datURLRedirected( const KURL& newDatURL );
    void signalChangeStatusbar( const QString& );
    void postResponse( const QString& response, const KURL& bbscgi = KURL() );
    //re-impl signals of KParts::Browserextension of KHTMLPart
    void openURLRequest( const KURL& url, const KParts::URLArgs& args = KParts::URLArgs() );
    void createNewWindow( const KURL& url, const KParts::URLArgs& args = KParts::URLArgs() );
    void setLocationBarURL( const QString& url );
    void enableAction( const char* name, bool enabled );
    void popupMenu( KXMLGUIClient*, const QPoint&, const KURL&, const QString&, mode_t );
    void signalThread( const Kita::Thread* );
    void writeSucceeded();
    void bookmarked( const QString& datURL, bool on );
    void openBoardRequested( const QString& URL, bool useTab );
};

namespace Kita
{
    // reimplement KActiveLabel based KTextBrowser
    class KitaSubjectLabel : public KTextBrowser
    {

    Q_OBJECT

    public:
        KitaSubjectLabel( QWidget *parent, const char * name = 0 );

        QSize minimumSizeHint() const;
        QSize sizeHint() const;
    private slots:
        void paletteChanged();
        void popupOpenAsNewTab();
        void popupCopyToClipboard();
        void popupFavorites();
    protected:
        virtual QPopupMenu * createPopupMenu ( const QPoint & pos );
        virtual void contentsMousePressEvent ( QMouseEvent * e );
        virtual void contentsMouseReleaseEvent ( QMouseEvent * e );
    signals:
        void urlMClick( const QString& url );
    private:
        QString m_popupURL;
    };

    class PopupTextBrowser : public QTextBrowser
    {
    public:
        PopupTextBrowser( QFrame* frame );
        ~PopupTextBrowser();
    };

    class ResPopup : public QFrame
    {
        KHTMLPart* m_part;
        Kita::PopupTextBrowser* m_browser;
    public:
        ResPopup( KHTMLView* view );
        ~ResPopup();
        void setText( QString& str );
        void adjustSize();
    };
}

#endif
